#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <mysql.h>
#include <sys/time.h>

#define DB_HOSTNAME "stuff"
#define DB_USERNAME "root"
#define DB_PASSWORD NULL
#define DB_DATABASE "test"

MYSQL* db_connect();
void db_disconnect(MYSQL *cn);
void db_do_query(MYSQL *cn, const char *query);

unsigned int g_read_timeout;

int main(int argc, char **argv)
{
	MYSQL *cn;
	char sqlstr[256];
	struct timeval  tv1, tv2;
	struct timezone tz1, tz2;

	if(argc != 2)
	{
		printf("arguments error.\n");
		printf("usage: ./read_timeout delay_in_seconds\n");
		exit(-1);
	}

	g_read_timeout = atoi(argv[1]);
	printf("\nread_timeout is set to %d sec.\n", g_read_timeout);

	strcpy(sqlstr, "select now()");

	printf("[enter] to connect."); getchar(); fflush(stdout);
	cn = db_connect("test");

	printf("[enter] after unplugging the network cable."); getchar(); fflush(stdout);

	gettimeofday(&tv1, &tz1);
	db_do_query(cn, sqlstr);
	gettimeofday(&tv2, &tz2);

	printf(":::: %ld msec. \n", (tv2.tv_sec-tv1.tv_sec)*1000+(tv2.tv_usec-tv1.tv_usec)/1000);

	mysql_close(cn);
	return 0;
}

static void die(MYSQL *cn, char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	putc('\n', stderr);
	exit(-1);
}

MYSQL* db_connect()
{
	unsigned int read_timeout = g_read_timeout;

	mysql_debug("d:t:0,/tmp/debug.out");

	MYSQL *cn = mysql_init(NULL);
	if (!cn) die(cn, "mysql_init failed: no memory");

	mysql_options(cn, MYSQL_OPT_READ_TIMEOUT, (void *)&read_timeout);

	if (!mysql_real_connect(cn, DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, 0, NULL, 0))
		die(cn, "mysql_real_connect failed: %s", mysql_error(cn));

	return(cn);
}

void db_do_query(MYSQL *cn, const char *query)
{
	if (mysql_query(cn, query) != 0)
		goto err;

	if (mysql_field_count(cn) > 0)
	{
		MYSQL_RES *res;
		MYSQL_ROW row, end_row;
		int num_fields;

		if (!(res = mysql_store_result(cn)))
			goto err;

		num_fields = mysql_num_fields(res);
		while ((row = mysql_fetch_row(res)))
		{
			for (end_row = row + num_fields; row < end_row; ++row)
			{
				printf("%s\t", row ? (char*)*row : "NULL");
			}
			fputc('\n', stdout);
		}
		fputc('\n', stdout);
		mysql_free_result(res);
	}
	else
	{
		printf("Affected rows: %lld\n", mysql_affected_rows(cn));
	}

	return;

err:
	fprintf(stderr, "db_do_query failed: %s [%s]\n", mysql_error(cn), query);
}

